"""
腾讯云客户端创建模块
"""
import os
from tencentcloud.common import credential
from tencentcloud.common.profile.client_profile import ClientProfile
from tencentcloud.common.profile.http_profile import HttpProfile
from tencentcloud.common.common_client import CommonClient
from tencentcloud.dnspod.v20210323 import dnspod_client
from tencentcloud.privatedns.v20201028 import privatedns_client
from tencentcloud.ecm.v20190719 import ecm_client

secret_id = os.getenv("TENCENTCLOUD_SECRET_ID")
secret_key = os.getenv("TENCENTCLOUD_SECRET_KEY")
default_region = os.getenv("TENCENTCLOUD_REGION")


def get_common_client(region: str = None, product = "dnspod", version="2021-03-23") -> CommonClient:
    """
    创建并返回通用客户端实例

    Args:
        region: 地域信息
        product: 产品名称
        version: 产品版本

    Returns:
        CommonClient: 通用客户端实例
    """
    cred = credential.Credential(secret_id, secret_key)
    if not region:
        region = default_region or "ap-guangzhou"

    http_profile = HttpProfile()
    http_profile.endpoint = "dnspod.tencentcloudapi.com"

    client_profile = ClientProfile()
    client_profile.httpProfile = http_profile
    client_profile.request_client = "MCP-Server"
    print("==========debug=======")
    print(secret_id)

    return CommonClient(product, version, cred, region, profile=client_profile)


def get_dnspod_client(region: str = None) -> dnspod_client.DnspodClient:
    """
    创建并返回DNSPod客户端

    Args:
        region: 地域信息

    Returns:
        CvmClient: CVM客户端实例
    """
    cred = credential.Credential(secret_id, secret_key)
    if not region:
        region = default_region or "ap-guangzhou"

    http_profile = HttpProfile()
    http_profile.endpoint = "dnspod.tencentcloudapi.com"

    client_profile = ClientProfile()
    client_profile.httpProfile = http_profile
    client_profile.request_client = "MCP-Server"

    return dnspod_client.DnspodClient(cred, region, client_profile)


def get_privatedns_client(region: str = None) -> privatedns_client.PrivatednsClient:
    """创建私有域"""
    cred = credential.Credential(secret_id, secret_key)

    if not region:
        region = default_region or "ap-guangzhou"

    # 实例化一个http选项，可选的，没有特殊需求可以跳过
    http_profile = HttpProfile()
    http_profile.endpoint = "privatedns.tencentcloudapi.com"

    # 实例化一个client选项，可选的，没有特殊需求可以跳过
    client_profile = ClientProfile()
    client_profile.httpProfile = http_profile
    # 实例化要请求产品的client对象,clientProfile是可选的
    return privatedns_client.PrivatednsClient(cred, region, client_profile)


def get_ecm_client(region: str = None) -> ecm_client.EcmClient:

    cred = credential.Credential(secret_id, secret_key)
    if not region:
        region = default_region or "ap-guangzhou"

    http_profile = HttpProfile()
    http_profile.endpoint = "ecm.tencentcloudapi.com"

    # 实例化一个client选项，可选的，没有特殊需求可以跳过
    client_profile = ClientProfile()
    client_profile.httpProfile = http_profile
    # 实例化要请求产品的client对象,clientProfile是可选的
    return ecm_client.EcmClient(cred, region, client_profile)



"""
腾讯云 privatedns 相关操作工具模块
"""
import json
from tencentcloud.common import credential
from tencentcloud.common.profile.client_profile import ClientProfile
from tencentcloud.common.profile.http_profile import HttpProfile
from tencentcloud.privatedns.v20201028 import privatedns_client, models as privatedns_models
from tencentcloud.ecm.v20190719 import ecm_client, models as ecm_models
from .capi_client import get_privatedns_client, get_ecm_client


def create_private_zone(action: str = None, version: str = None, domain: str = None,
                        remark: str = None, dns_forward_status: str = None, cname_speedup_status: str = None) -> str:
    client = get_privatedns_client()
    # 实例化一个请求对象,每个接口都会对应一个request对象
    req = privatedns_models.CreatePrivateZoneRequest()
    params = {}
    if action is not None:
        params["Action"] = action
    if version is not None:
        params["Version"] = version
    if domain is not None:
        params["Domain"] = domain
    if remark is not None:
        params["Remark"] = remark
    if dns_forward_status is not None:
        params["DnsForwardStatus"] = dns_forward_status
    if cname_speedup_status is not None:
        params["CnameSpeedupStatus"] = cname_speedup_status

    req.from_json_string(json.dumps(params))

    # 返回的resp是一个CreatePrivateZoneResponse的实例，与请求对象对应
    resp = client.CreatePrivateZone(req)
    # 输出json格式的字符串回包
    return resp.to_json_string()




def describe_private_zone_list(action: str = None, version: str = None, offset: int = None,
                               limit: int = None, filters: list[object] = None) -> str:
    client = get_privatedns_client()
    # 实例化一个请求对象,每个接口都会对应一个request对象
    req = privatedns_models.DescribePrivateZoneListRequest()
    params = {}
    if action is not None:
        params["Action"] = action
    if version is not None:
        params["Version"] = version
    if offset is not None:
        params["Offset"] = offset
    if limit is not None:
        params["Limit"] = limit
    if filters is not None:
        params["Filters"] = filters

    req.from_json_string(json.dumps(params))

    # 返回的resp是一个CreatePrivateZoneResponse的实例，与请求对象对应
    resp = client.DescribePrivateZoneList(req)
    # 输出json格式的字符串回包
    return resp.to_json_string()



def describe_private_zone(action: str = None, version: str = None, zone_id: str = None) -> str:
    client = get_privatedns_client()
    # 实例化一个请求对象,每个接口都会对应一个request对象
    req = privatedns_models.DescribePrivateZoneRequest()
    params = {}
    if action is not None:
        params["Action"] = action
    if version is not None:
        params["Version"] = version
    if zone_id is not None:
        params["ZoneId"] = zone_id

    req.from_json_string(json.dumps(params))

    # 返回的resp是一个CreatePrivateZoneResponse的实例，与请求对象对应
    resp = client.DescribePrivateZone(req)
    # 输出json格式的字符串回包
    return resp.to_json_string()





def describe_private_zone_record_list(action: str = None, version: str = None, zone_id: str = None, filters: list[object] = None, offset: int = None, limit: int = None) -> str:
    client = get_privatedns_client()
    req = privatedns_models.DescribePrivateZoneRecordListRequest()
    params = {}
    if action is not None:
        params["Action"] = action
    if version is not None:
        params["Version"] = version
    if zone_id is not None:
        params["ZoneId"] = zone_id
    if filters is not None:
        params["Filters"] = filters
    if offset is not None:
        params["Offset"] = offset
    if limit is not None:
        params["Limit"] = limit
    req.from_json_string(json.dumps(params))
    resp = client.DescribePrivateZoneRecordList(req)
    return resp.to_json_string()



def describe_request_data(action: str = None, version: str = None, time_range_begin: str = None, filters: list[object] = None,
                          time_range_end: str = None, export: str = None):
    client = get_privatedns_client()
    req = privatedns_models.DescribeRequestDataRequest()
    params = {}
    if action is not None:
        params["Action"] = action
    if version is not None:
        params["Version"] = version
    if time_range_begin is not None:
        params["TimeRangeBegin"] = time_range_begin
    if filters is not None:
        params["Filters"] = filters
    if time_range_end is not None:
        params["TimeRangeEnd"] = time_range_end
    if export is not None:
        params["Export"] = export
    req.from_json_string(json.dumps(params))
    resp = client.DescribeRequestData(req)
    return resp.to_json_string()



"""
腾讯云 DNSPod 服务主模块
"""
from asyncio.log import logger
from typing import Any
from mcp.server.models import InitializationOptions
import mcp.types as types
from mcp.server.lowlevel import NotificationOptions, Server
import mcp.server.stdio
from . import tool_dnspod, tool_privatedns

server = Server("privatedns")


@server.call_tool()
async def handle_call_tool(
        name: str, arguments: dict[str, Any] | None
) -> list[types.TextContent | types.ImageContent | types.EmbeddedResource]:
    """处理工具调用请求"""
    try:
        result = None
        if name == "DescribeDomainList":
            result = tool_dnspod.describe_domain_list(
                type=arguments.get("type"),
                offset=arguments.get("offset"),
                limit=arguments.get("limit"),
                groupId=arguments.get("groupId"),
                keyword=arguments.get("keyword"),
                tags=arguments.get("tags"),
            )
        if name == "CreatePrivateZone":
            result = tool_privatedns.create_private_zone(
                action=arguments.get("Action"),
                version=arguments.get("Version"),
                domain=arguments.get("Domain"),
                remark=arguments.get("Remark"),
                dns_forward_status=arguments.get("DnsForwardStatus"),
                cname_speedup_status=arguments.get("CnameSpeedupStatus")
            )
        if name == "DescribePrivateZoneList":
            result = tool_privatedns.describe_private_zone_list(
                action=arguments.get("Action"),
                version=arguments.get("Version"),
                offset=arguments.get("Offset"),
                limit=arguments.get("Limit"),
                filters=arguments.get("Filters.N"),
            )
        if name == "DescribePrivateZone":
            result = tool_privatedns.describe_private_zone(
                action=arguments.get("Action"),
                version=arguments.get("Version"),
                zone_id=arguments.get("ZoneId"),
            )
        if name == "DescribePrivateZoneRecordList":
            result = tool_privatedns.describe_private_zone_record_list(
                action=arguments.get("Action"),
                version=arguments.get("Version"),
                zone_id=arguments.get("ZoneId"),
                offset=arguments.get("Offset"),
                limit=arguments.get("Limit"),
                filters=arguments.get("Filters.N"),
            )
        if name == "DescribeRequestData":
            result = tool_privatedns.describe_request_data(
                action=arguments.get("Action"),
                version=arguments.get("Version"),
                time_range_begin=arguments.get("TimeRangeBegin"),
                filters=arguments.get("Filters.N"),
                time_range_end=arguments.get("TimeRangeEnd"),
                export=arguments.get("Export"),
            )

        return [types.TextContent(type="text", text=str(result))]

    except Exception as e:
        return [types.TextContent(type="text", text=f"错误: {str(e)}")]


@server.list_tools()
async def handle_list_tools() -> list[types.Tool]:
    return [
        types.Tool(
            name="DescribeDomainList",
            description="查询DNSPod的域名列表",
            inputSchema={
                "type": "object",
                "properties": {
                    "Type": {
                        "type": "string",
                        "description": "域名分组类型，默认为ALL。可取值为ALL，MINE，SHARE，ISMARK，PAUSE，VIP，RECENT，SHARE_OUT，FREE。示例值：ALL",
                    },
                    "Offset": {
                        "type": "integer",
                        "description": "记录开始的偏移, 第一条记录为 0, 依次类推。默认值为0。示例值：0",
                    },
                    "Limit": {
                        "type": "integer",
                        "description": "要获取的域名数量, 比如获取20个, 则为20。默认值为3000。示例值：20",
                    },
                    "GroupId": {
                        "type": "integer",
                        "description": "分组ID, 获取指定分组的域名。示例值：1",
                    },
                    "Keyword": {
                        "type": "string",
                        "description": "根据关键字搜索域名。示例值：qq",
                    },
                    "Tags": {
                        "type": "array",
                        "items": {"type": "object"},
                        "description": "标签过滤",
                    },
                },
            },
        ),
        types.Tool(
            name="CreatePrivateZone",
            description="创建一个私有域",
            inputSchema={
                "type": "object",
                "properties": {
                    "Action":  {
                        "type": "string",
                        "description": "Action 值为CreatePrivateZone",
                    },
                    "Version":  {
                        "type": "string",
                        "description": "Version 值为2020-10-28",
                    },
                    "Domain":  {
                        "type": "string",
                        "description": "域名，格式必须是标准的TLD。示例值：a.com",
                    },
                    "Remark": {
                        "type": "string",
                        "description": "备注, 示例值：测试域名",
                    },
                    "DnsForwardStatus": {
                        "type": "string",
                        "description": "是否开启子域名递归：ENABLED， DISABLED。默认值为ENABLED。示例值：ENABLED",
                    },
                    "CnameSpeedupStatus	": {
                        "type": "string",
                        "description": "是否CNAME加速：ENABLED，DISABLED",
                        "default": "ENABLED",
                    },
                },
                "required": ["Action", "Version", "Domain"],
            },
        ),
        types.Tool(
            name="DescribePrivateZoneList",
            description="获取私有域列表",
            inputSchema={
                "type": "object",
                "properties": {
                    "Action":  {
                        "type": "string",
                        "description": "Action 值为DescribePrivateZoneList",
                    },
                    "Version":  {
                        "type": "string",
                        "description": "Version 值为2020-10-28",
                    },
                    "Offset":  {
                        "type": "integer",
                        "description": "记录开始的偏移, 第一条记录为 0, 依次类推。",
                    },
                    "Limit":  {
                        "type": "integer",
                        "description": "分页限制数目， 最大100，默认20。",
                        "default": 20,
                    },
                    "Filters.N":  {
                        "type": "array",
                        "items": {
                            "type": "object",
                            "properties": {
                                "Name": {"type": "string", "description": "过滤参数名称,示例值：Domain"},
                                "Values": {"type": "array", "items": {"type": "string"},
                                           "description": "过滤参数值数组,示例值：qq.com"},
                            },
                            "required": ["Name", "Values"],
                        },
                        "description": "过滤参数",
                    }
                },
                "required": ["Action", "Version"],
            },
        ),
        types.Tool(
            name="DescribePrivateZone",
            description="获取私有域信息",
            inputSchema={
                "type": "object",
                "properties": {
                    "Action": {
                        "type": "string",
                        "description": "Action 值为DescribePrivateZone",
                    },
                    "Version": {
                        "type": "string",
                        "description": "Version 值为2020-10-28",
                    },
                    "ZoneId": {
                        "type": "string",
                        "description": "私有域ID",
                    },
                },
                "required": ["Action", "Version", "ZoneId"],
            },
        ),
        types.Tool(
            name="DescribePrivateZoneRecordList",
            description="获取私有域记录列表",
            inputSchema={
                "type": "object",
                "properties": {
                    "Action": {
                        "type": "string",
                        "description": "Action 值为DescribePrivateZoneRecordList",
                    },
                    "Version": {
                        "type": "string",
                        "description": "Version 值为2020-10-28",
                    },
                    "ZoneId": {
                        "type": "string",
                        "description": "私有域ID",
                    },
                    "Filters.N": {
                        "type": "array",
                        "items": {
                            "type": "object",
                            "properties": {
                                "Name": {
                                    "type": "string",
                                    "description": "过滤条件名称",
                                },
                                "Values": {"type": "array", "items": {"type": "string"}, "description": "过滤条件值"}
                            },
                            "required": ["Name", "Values"],
                        },
                    },
                    "Offset": {
                        "type": "integer",
                        "description": "偏移量。",
                    },
                    "Limit": {
                        "type": "integer",
                        "description": "返回数量",
                    },
                },
                "required": ["Action", "Version", "ZoneId"],
            },
        ),
        types.Tool(
            name="DescribeRequestData",
            description="获取私有域解析请求量",
            inputSchema={
                "type": "object",
                "properties": {
                    "Action": {
                        "type": "string",
                        "description": "Action 值为DescribeRequestData",
                    },
                    "Version": {
                        "type": "string",
                        "description": "Version 值为2020-10-28",
                    },
                    "TimeRangeBegin": {
                        "type": "string",
                        "description": "请求量统计起始时间, 格式为yyyy-mm-dd hh:mm:ss, 默认是当前这一天的开始时间",
                    },
                    "Filters.N": {
                        "type": "array",
                        "items": {
                            "type": "object",
                            "properties": {
                                "Name": {"type": "string", "description": "过滤参数名称,示例值：Domain"},
                                "Values": {"type": "array", "items": {"type": "string"},
                                           "description": "过滤参数值数组,示例值：qq.com"},
                            },
                            "required": ["Name", "Values"],
                        },
                        "description": "过滤参数",
                    },
                    "TimeRangeEnd": {
                        "type": "string",
                        "description": "请求量统计结束时间, 格式为yyyy-mm-dd hh:mm:ss",
                    },
                    "Export": {
                        "type": "boolean",
                        "description": "是否导出数据,true: 导出,false: 不导出",
                    }
                },
                "required": ["Action", "Version", "TimeRangeBegin"],
            }
        )
    ]


async def serve():
    async with mcp.server.stdio.stdio_server() as (read_stream, write_stream):
        logger.info("Server running with stdio transport")

        await server.run(
            read_stream,
            write_stream,
            InitializationOptions(
                server_name="privatedns",
                server_version="1.0.0",
                capabilities=server.get_capabilities(
                    notification_options=NotificationOptions(),
                    experimental_capabilities={},
                ),
            ),
        )



"""
腾讯云 DNSPod 相关操作工具模块
"""
import json
from tencentcloud.dnspod.v20210323 import dnspod_client, models as dnspod_models
from .capi_client import get_dnspod_client, get_common_client
from asyncio.log import logger
from tencentcloud.common.exception.tencent_cloud_sdk_exception import TencentCloudSDKException
import random
import string
import re
from typing import Optional, Tuple

def describe_domain_list(type: str=None, offset: int=None, limit: int=None, groupId: int=None, keyword: str=None, tags: list=None) -> str:
    """查询域名列表"""
    client = get_dnspod_client()  # 使用默认地域
    req = dnspod_models.DescribeDomainListRequest()

    params = {}
    if type is not None:
        params["Type"] = type
    if offset is not None:
        params["Offset"] = offset
    if limit is not None:
        params["Limit"] = limit
    if groupId is not None:
        params["GroupId"] = groupId
    if keyword is not None:
        params["Keyword"] = keyword
    if tags is not None:
        params["Tags"] = tags
    
    req.from_json_string(json.dumps(params))
    resp = client.DescribeDomainList(req)

    return resp.to_json_string()